@@ -33,6 +33,7 @@
 
 typedef struct {
     AtomPtr ghostview;
+    AtomPtr gv_colors;
     AtomPtr next;
     AtomPtr page;
     AtomPtr done;
@@ -57,6 +58,7 @@ struct record_list {
 
 typedef struct {
     /* resources */
+    Pixel		foreground;
     Cursor		cursor;
     Cursor		busy_cursor;
     XtCallbackList	callback;
@@ -64,10 +66,12 @@ typedef struct {
     XtCallbackList	output_callback;
     String		interpreter;
     Boolean		quiet;
+    Boolean		safer;
     Boolean		use_bpixmap;
-    String		preload;
+    String		arguments;
     String		filename;
     XtPageOrientation	orientation;
+    XtPalette		palette;
     float		xdpi;
     float		ydpi;
     int			llx;
