@@ -431,20 +431,11 @@ track_and_zoom(w, client_data, call_data)
     /* If the file changed, cannot zoom */
     stat(filename, &sbuf);
     if (mtime != sbuf.st_mtime) return;
-							num_args = 0;
-    XtSetArg(args[num_args], XtNcolormap, cmap);	num_args++;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
     zoom = XtCreatePopupShell("zoom", topLevelShellWidgetClass,
-			      toplevel, args, num_args);
+			      toplevel, NULL, ZERO);
 
-							num_args = 0;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
     zoomform = XtCreateManagedWidget("form", formWidgetClass,
-				     zoom, args, num_args);
+				     zoom, NULL, ZERO);
 
     llx = p->psx - p->width/2;
     lly = p->psy - p->height/2;
@@ -480,9 +471,6 @@ track_and_zoom(w, client_data, call_data)
     top_margin = current_ury - ury;
 
 							num_args = 0;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
     XtSetArg(args[num_args], XtNtop, XtChainTop);	num_args++;
     XtSetArg(args[num_args], XtNbottom, XtChainBottom);	num_args++;
     XtSetArg(args[num_args], XtNleft, XtChainLeft);	num_args++;
@@ -516,10 +504,6 @@ track_and_zoom(w, client_data, call_data)
 		  (XtPointer)zoompage);
 
 							num_args = 0;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
     XtSetArg(args[num_args], XtNfromVert, zoompage);	num_args++;
     XtSetArg(args[num_args], XtNtop, XtChainBottom);	num_args++;
     XtSetArg(args[num_args], XtNbottom, XtChainBottom);	num_args++;
