@@ -474,7 +474,7 @@ try_try_again(client_data, timer)
     XtPointer client_data;
     XtIntervalId *timer;
 {
-    XSync(XtDisplay(toplevel), False);
+    XSync(XtDisplay(toplevel), False);	/* Push everything out */
     if (XtAppPending(app_con)) {
 	XtAppAddTimeOut(app_con, delay, try_try_again, NULL);
 	/* fprintf(stderr, "Delaying(%d)...\n",delay); */
@@ -494,10 +494,6 @@ void
 layout_ghostview()
 {
     Arg args[20];
-    Cardinal num_args;
-    Widget w;
-    /* Yuck, cannot get vScrollbar via the usual methods */
-    TextWidget tw;
     Dimension min_width, min_height;
     Dimension max_width, max_height;
     Dimension form_width, form_height;
@@ -667,41 +663,6 @@ layout_ghostview()
     }
 
     if (correct) {
-	if (special_cmap) {
-	    /* Yuck, cannot get vScrollbar via the usual methods */
-	    /* w = XtNameToWidget(toc, "*vScrollbar"); */
-	    tw = (TextWidget) toc;
-	    w = tw->text.vbar;
-	    if (w) {
-		/* Double Yuck, have to set them twice to make them */
-		/* get the right colors */
-								num_args = 0;
-		XtSetArg(args[num_args], XtNforeground, white);	num_args++;
-		XtSetArg(args[num_args], XtNbackground, black);	num_args++;
-		XtSetValues(w, args, num_args);
-								num_args = 0;
-		XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-		XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-		XtSetValues(w, args, num_args);
-	    }
-
-	    w = XtNameToWidget(pageview, "horizontal");
-	    if (w) {
-								num_args = 0;
-		XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-		XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-		XtSetValues(w, args, num_args);
-	    }
-
-	    w = XtNameToWidget(pageview, "vertical");
-	    if (w) {
-								num_args = 0;
-		XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-		XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-		XtSetValues(w, args, num_args);
-	    }
-	}
-
 	set_size_hints(min_width, min_height, min_width+page_width,
 		       max(form_height,
 			   page_height + 2*(view_border + distance)));
@@ -722,118 +683,6 @@ layout_ghostview()
 
 }
 
-/* Create a Standard colormap for ghostscript to use. */
-void
-SetStandardColormap(w)
-    Widget w;
-{
-    XVisualInfo xvinfo;
-    XVisualInfo *xvinfop;
-    Atom prop;
-    XStandardColormap *std_cmap = NULL;
-    XStandardColormap *scmap, *sp;
-    Screen *scr = DefaultScreenOfDisplay(XtDisplay(w));
-    int nitems, i;
-    Boolean has_color;
-    Arg args[20];
-    Cardinal num_args;
-
-    XtSetArg(args[0], XtNvisual, &(xvinfo.visual));
-    XtGetValues(w, args, ONE);
-    if (xvinfo.visual == CopyFromParent)
-	xvinfo.visual = DefaultVisualOfScreen(scr);
-    xvinfo.visualid = XVisualIDFromVisual(xvinfo.visual);
-    xvinfop = XGetVisualInfo(XtDisplay(w), VisualIDMask, &xvinfo, &nitems);
-    if (xvinfop == NULL) {
-        fprintf(stderr, "Bad visual class.\n");
-	exit(1);
-    }
-    xvinfo = *xvinfop;
-    XtFree((char *)xvinfop);
-
-    has_color = (xvinfo.class != StaticGray) && (xvinfo.class != GrayScale);
-
-    if (has_color) {
-	if (xvinfo.visual == DefaultVisualOfScreen(scr)) {
-	    prop = XA_RGB_DEFAULT_MAP;
-	} else {
-	    prop = XA_RGB_BEST_MAP;
-	}
-    } else {
-	prop = XA_RGB_GRAY_MAP;
-    }
-
-    if (XGetRGBColormaps(XtDisplay(w), RootWindowOfScreen(scr),
-			 &scmap, &nitems, prop)) {
-	for (i = 0, sp = scmap; i < nitems; i++, sp++) {
-	    if (xvinfo.visualid == sp->visualid) {
-		std_cmap = sp;
-		break;
-	    }
-	}
-    }
-
-    if (!std_cmap && app_res.install_std_cmap) {
-	if (XmuLookupStandardColormap(XtDisplay(w),
-				      DefaultScreen(XtDisplay(w)),
-				      xvinfo.visualid, xvinfo.depth,
-				      prop, False, app_res.retain_std_cmap)) {
-	    if (XGetRGBColormaps(XtDisplay(w), RootWindowOfScreen(scr),
-				 &scmap, &nitems, prop)) {
-		for (i = 0, sp = scmap; i < nitems; i++, sp++) {
-		    if (xvinfo.visualid == sp->visualid) {
-			std_cmap = sp;
-		    }
-		}
-	    }
-	}
-    }
-
-    if (std_cmap && xvinfo.visual == DefaultVisualOfScreen(scr) &&
-	std_cmap->colormap != DefaultColormapOfScreen(scr) &&
-	!app_res.private_cmap) {
-	std_cmap = NULL;
-    }
-    if (std_cmap) {
-	cmap = std_cmap->colormap;
-	if (std_cmap->colormap != DefaultColormapOfScreen(scr)) {
-	    black = std_cmap->base_pixel;
-	    white = std_cmap->red_max * std_cmap->red_mult +
-		    std_cmap->green_max * std_cmap->green_mult +
-		    std_cmap->blue_max * std_cmap->blue_mult +
-		    std_cmap->base_pixel;
-	} else {
-	    white = WhitePixelOfScreen(scr);
-	    black = BlackPixelOfScreen(scr);
-	}
-    } else {
-	if (xvinfo.visual == DefaultVisualOfScreen(scr)) {
-	    cmap = DefaultColormapOfScreen(scr);
-	    white = WhitePixelOfScreen(scr);
-	    black = BlackPixelOfScreen(scr);
-	} else {
-	    XColor c;
-	    cmap = XCreateColormap(XtDisplay(w), RootWindowOfScreen(scr),
-				   xvinfo.visual, AllocNone);
-	    c.red = c.green = c.blue = ~(unsigned short)0;
-	    XAllocColor(XtDisplay(w), cmap, &c);
-	    white = c.pixel;
-	    c.red = c.green = c.blue = 0;
-	    XAllocColor(XtDisplay(w), cmap, &c);
-	    black = c.pixel;
-	}
-    }
-    special_cmap = cmap != DefaultColormapOfScreen(scr);
-
-							num_args = 0;
-    XtSetArg(args[num_args], XtNcolormap, cmap);	num_args++;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
-    XtSetValues(w, args, num_args);
-}
-
 /* Compute new dpi from magstep */
 void
 magnify(dpi, magstep)
@@ -1347,13 +1196,8 @@ build_pagemedia_menu()
     if (pagemediamenu) XtDestroyWidget(pagemediamenu);
     force_document_media = False;
 
-							num_args = 0;
-    XtSetArg(args[num_args], XtNcolormap, cmap);	num_args++;
-    if (special_cmap) {
-	XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-    }
     pagemediamenu = XtCreatePopupShell("menu", simpleMenuWidgetClass,
-				       pagemediabutton, args, num_args);
+				       pagemediabutton, NULL, ZERO);
 
     /* Build the Page Media menu */
     /* the Page media menu has two parts.
@@ -1369,10 +1213,6 @@ build_pagemedia_menu()
     if (doc && doc->nummedia) {
 	for (i = 0; i < doc->nummedia; i++) {
 								num_args = 0;
-	    if (special_cmap) {
-		XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-		XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-	    }
 	    XtSetArg(args[num_args], XtNleftMargin, 20);	num_args++;
 	    pagemediaentry[i] = XtCreateManagedWidget(doc->media[i].name,
 				smeBSBObjectClass, pagemediamenu,
@@ -1382,10 +1222,6 @@ build_pagemedia_menu()
 	}
 
 							num_args = 0;
-	if (special_cmap) {
-	    XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-	    XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-	}
 	w = XtCreateManagedWidget("line", smeLineObjectClass, pagemediamenu,
 				  args, num_args);
     }
@@ -1400,10 +1236,6 @@ build_pagemedia_menu()
 	    }
 	}
 							num_args = 0;
-	if (special_cmap) {
-	    XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-	    XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-	}
 	XtSetArg(args[num_args], XtNleftMargin, 20);	num_args++;
 	pagemediaentry[i+base_papersize] = XtCreateManagedWidget(
 					    papersizes[i].name,
@@ -1424,18 +1256,10 @@ build_label_menu(parent, name, label, bitmap)
     Cardinal num_args;
     Widget menu, entry;
 								num_args = 0;
-	XtSetArg(args[num_args], XtNcolormap, cmap);		num_args++;
-	if (special_cmap) {
-	    XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-	}
 	menu = XtCreatePopupShell("menu", simpleMenuWidgetClass,
 				  parent, args, num_args);
 
 								num_args = 0;
-	if (special_cmap) {
-	    XtSetArg(args[num_args], XtNforeground, black);	num_args++;
-	    XtSetArg(args[num_args], XtNbackground, white);	num_args++;
-	}
 	XtSetArg(args[num_args], XtNlabel, label);		num_args++;
 	if (bitmap) {
 	    XtSetArg(args[num_args], XtNleftMargin, 20);	num_args++;
