@@ -42,6 +42,7 @@
 /* can only be used with character constants */
 #define length(a) (sizeof(a)-1)
 #define iscomment(a, b)	(strncmp(a, b, length(b)) == 0)
+#define DSCcomment(a) (a[0] == '%' && a[1] == '%')
 
 	/* list of standard paper sizes from Adobe's PPD. */
 
@@ -69,7 +70,7 @@ struct documentmedia papersizes[] = {
 static char *readline();
 static char *gettextline();
 static char *gettext();
-static int  isblank();
+static int  blank();
 
 /*
  *	psscan -- scan the PostScript file for document structuring comments.
@@ -105,11 +106,11 @@ static int  isblank();
  *      %%PageOrder: Ascend|Descend|Special|(atend)
  *      %%Title: <textline>
  *      %%DocumentMedia: <text> <real> <real> <real> <text> <text>
- *      %%DocumentPageSizes: <text>
+ *      %%DocumentPaperSizes: <text>
  *      %%EndComments
  *
  *      Note: Either the 3.0 or 2.0 syntax for %%Pages is accepted.
- *            Also either the 2.0 %%DocumentPageSizes or the 3.0
+ *            Also either the 2.0 %%DocumentPaperSizes or the 3.0
  *            %%DocumentMedia comments are accepted as well.
  *
  *      The header section ends either explicitly with %%EndComments or
@@ -145,7 +146,7 @@ static int  isblank();
  *
  *      The Setup section is where the version 2 page defaults are found.
  *      This section either explicitly begins with %%BeginSetup or implicitly
- *      with any noblank line after the Prolog.
+ *      with any nonblank line after the Prolog.
  *
  *      %%BeginSetup
  *      %%PageBoundingBox: <int> <int> <int> <int>
@@ -203,6 +204,7 @@ psscan(file)
     char line[PSLINELENGTH];	/* 255 characters + 1 newline + 1 NULL */
     char text[PSLINELENGTH];	/* Temporary storage for text */
     long position;		/* Position of the current line */
+    long beginsection;		/* Position of the beginning of the section */
     unsigned int line_len; 	/* Length of the current line */
     unsigned int section_len;	/* Place to accumulate the section length */
     char *next_char;		/* 1st char after text returned by gettext() */
@@ -236,12 +238,12 @@ psscan(file)
     while (preread || readline(line, sizeof line, file, &position, &line_len)) {
 	if (!preread) section_len += line_len;
 	preread = 0;
-	if (iscomment(line, "%%EndComments") ||
-	    line[0] != '%' ||
-	    (!isprint(line[1]) || line[1] == ' ' ||
-	     line[1] == '\t' || line[1] == '\n')) {
+	if (line[0] != '%' ||
+	    iscomment(line+1, "%EndComments") ||
+	    line[1] == ' ' || line[1] == '\t' || line[1] == '\n' ||
+	    !isprint(line[1])) {
 	    break;
-	} else if (!iscomment(line, "%%")) {
+	} else if (line[1] != '%') {
 	    /* Do nothing */
 	} else if (doc->title == NULL && iscomment(line+2, "Title:")) {
 	    doc->title = gettextline(line+length("%%Title:"));
@@ -325,12 +327,14 @@ psscan(file)
 			    }
 			}
 		    case 1:
-			doc->pages = (struct page *) calloc(maxpages,
-							sizeof(struct page));
-			if (doc->pages == NULL) {
-			    fprintf(stderr,
+			if (maxpages > 0) {
+			    doc->pages = (struct page *) calloc(maxpages,
+							   sizeof(struct page));
+			    if (doc->pages == NULL) {
+				fprintf(stderr,
 				    "Fatal Error: Dynamic memory exhausted.\n");
-			    exit(-1);
+				exit(-1);
+			    }
 			}
 		}
 	    }
@@ -357,7 +361,7 @@ psscan(file)
 	    }
 	    preread=1;
 	    while (readline(line, sizeof line, file, &position, &line_len) &&
-		   iscomment(line, "%%+")) {
+		   DSCcomment(line) && iscomment(line+2, "+")) {
 		section_len += line_len;
 		doc->media = (struct documentmedia *)
 			     realloc(doc->media,
@@ -462,7 +466,7 @@ psscan(file)
 	    }
 	    preread=1;
 	    while (readline(line, sizeof line, file, &position, &line_len) &&
-		   iscomment(line, "%%+")) {
+		   DSCcomment(line) && iscomment(line+2, "+")) {
 		section_len += line_len;
 		next_char = line + length("%%+");
 		while (cp = gettext(next_char, &next_char)) {
@@ -501,7 +505,7 @@ psscan(file)
 	}
     }
 
-    if (iscomment(line, "%%EndComments")) {
+    if (DSCcomment(line) && iscomment(line+2, "EndComments")) {
 	readline(line, sizeof line, file, &position, &line_len);
 	section_len += line_len;
     }
@@ -510,15 +514,18 @@ psscan(file)
 
     /* Optional Preview comments for encapsulated PostScript files */ 
 
-    while (isblank(line) &&
+    beginsection = position;
+    section_len = line_len;
+    while (blank(line) &&
 	   readline(line, sizeof line, file, &position, &line_len)) {
+	section_len += line_len;
     }
 
-    if (doc->epsf && iscomment(line, "%%BeginPreview")) {
-	doc->beginpreview = position;
-	section_len = line_len;
+    if (doc->epsf && DSCcomment(line) && iscomment(line+2, "BeginPreview")) {
+	doc->beginpreview = beginsection;
+	beginsection = 0;
 	while (readline(line, sizeof line, file, &position, &line_len) &&
-	       !iscomment(line, "%%EndPreview")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndPreview"))) {
 	    section_len += line_len;
 	}
 	section_len += line_len;
@@ -530,17 +537,22 @@ psscan(file)
 
     /* Page Defaults for Version 3.0 files */
 
-    while (isblank(line) &&
+    if (beginsection == 0) {
+	beginsection = position;
+	section_len = line_len;
+    }
+    while (blank(line) &&
 	   readline(line, sizeof line, file, &position, &line_len)) {
+	section_len += line_len;
     }
 
-    if (iscomment(line, "%%BeginDefaults")) {
-	doc->begindefaults = position;
-	section_len = line_len;
+    if (DSCcomment(line) && iscomment(line+2, "BeginDefaults")) {
+	doc->begindefaults = beginsection;
+	beginsection = 0;
 	while (readline(line, sizeof line, file, &position, &line_len) &&
-	       !iscomment(line, "%%EndDefaults")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndDefaults"))) {
 	    section_len += line_len;
-	    if (!iscomment(line, "%%")) {
+	    if (!DSCcomment(line)) {
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
@@ -599,27 +611,37 @@ psscan(file)
 
     /* Document Prolog */
 
-    while (isblank(line) &&
+    if (beginsection == 0) {
+	beginsection = position;
+	section_len = line_len;
+    }
+    while (blank(line) &&
 	   readline(line, sizeof line, file, &position, &line_len)) {
+	section_len += line_len;
     }
 
-    if (!iscomment(line, "%%BeginSetup") && !iscomment(line, "%%Page:") &&
-	!iscomment(line, "%%Trailer") && !iscomment(line, "%%EOF")) {
-	doc->beginprolog = position;
-	section_len = line_len;
+    if (!(DSCcomment(line) &&
+	  (iscomment(line+2, "BeginSetup") ||
+	   iscomment(line+2, "Page:") ||
+	   iscomment(line+2, "Trailer") ||
+	   iscomment(line+2, "EOF")))) {
+	doc->beginprolog = beginsection;
+	beginsection = 0;
 	preread = 1;
 
 	while ((preread ||
 		readline(line, sizeof line, file, &position, &line_len)) &&
-	       !iscomment(line, "%%EndProlog") &&
-	       !iscomment(line, "%%BeginSetup") &&
-	       !iscomment(line, "%%Page:") && !iscomment(line, "%%Trailer") &&
-	       !iscomment(line, "%%EOF")) {
+	       !(DSCcomment(line) &&
+	         (iscomment(line+2, "EndProlog") ||
+	          iscomment(line+2, "BeginSetup") ||
+	          iscomment(line+2, "Page:") ||
+	          iscomment(line+2, "Trailer") ||
+	          iscomment(line+2, "EOF")))) {
 	    if (!preread) section_len += line_len;
 	    preread = 0;
 	}
 	section_len += line_len;
-	if (iscomment(line, "%%EndProlog")) {
+	if (DSCcomment(line) && iscomment(line+2, "EndProlog")) {
 	    readline(line, sizeof line, file, &position, &line_len);
 	    section_len += line_len;
 	}
@@ -629,22 +651,32 @@ psscan(file)
 
     /* Document Setup,  Page Defaults found here for Version 2 files */
 
-    while (isblank(line) &&
+    if (beginsection == 0) {
+	beginsection = position;
+	section_len = line_len;
+    }
+    while (blank(line) &&
 	   readline(line, sizeof line, file, &position, &line_len)) {
+	section_len += line_len;
     }
 
-    if (!iscomment(line, "%%Page:") && !iscomment(line, "%%Trailer") &&
-	!iscomment(line, "%%EOF")) {
-	doc->beginsetup = position;
-	section_len = line_len;
+    if (!(DSCcomment(line) &&
+	  (iscomment(line+2, "Page:") ||
+	   iscomment(line+2, "Trailer") ||
+	   iscomment(line+2, "EOF")))) {
+	doc->beginsetup = beginsection;
+	beginsection = 0;
 	preread = 1;
 	while ((preread ||
 		readline(line, sizeof line, file, &position, &line_len)) &&
-	       !iscomment(line, "%%EndSetup") && !iscomment(line, "%%Page:") &&
-	       !iscomment(line, "%%Trailer") && !iscomment(line, "%%EOF")) {
+	       !(DSCcomment(line) &&
+	         (iscomment(line+2, "EndSetup") ||
+	          iscomment(line+2, "Page:") ||
+	          iscomment(line+2, "Trailer") ||
+	          iscomment(line+2, "EOF")))) {
 	    if (!preread) section_len += line_len;
 	    preread = 0;
-	    if (!iscomment(line, "%%")) {
+	    if (!DSCcomment(line)) {
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
@@ -699,7 +731,7 @@ psscan(file)
 	    }
 	}
 	section_len += line_len;
-	if (iscomment(line, "%%EndSetup")) {
+	if (DSCcomment(line) && iscomment(line+2, "EndSetup")) {
 	    readline(line, sizeof line, file, &position, &line_len);
 	    section_len += line_len;
 	}
@@ -709,12 +741,17 @@ psscan(file)
 
     /* Individual Pages */
 
-    while (isblank(line) &&
+    if (beginsection == 0) {
+	beginsection = position;
+	section_len = line_len;
+    }
+    while (blank(line) &&
 	   readline(line, sizeof line, file, &position, &line_len)) {
+	section_len += line_len;
     }
 
 newpage:
-    while (iscomment(line, "%%Page:")) {
+    while (DSCcomment(line) && iscomment(line+2, "Page:")) {
 	if (maxpages == 0) {
 	    maxpages = 1;
 	    doc->pages = (struct page *) calloc(maxpages, sizeof(struct page));
@@ -747,14 +784,21 @@ newpage:
 	memset(&(doc->pages[doc->numpages]), 0, sizeof(struct page));
 	page_bb_set = NONE;
 	doc->pages[doc->numpages].label = label;
-	doc->pages[doc->numpages].begin = position;
-	section_len = line_len;
+	if (beginsection) {
+	    doc->pages[doc->numpages].begin = beginsection;
+	    beginsection = 0;
+	} else {
+	    doc->pages[doc->numpages].begin = position;
+	    section_len = line_len;
+	}
 continuepage:
 	while (readline(line, sizeof line, file, &position, &line_len) &&
-	       !iscomment(line, "%%Page:") && !iscomment(line, "%%Trailer") &&
-	       !iscomment(line, "%%EOF")) {
+	       !(DSCcomment(line) &&
+	         (iscomment(line+2, "Page:") ||
+	          iscomment(line+2, "Trailer") ||
+	          iscomment(line+2, "EOF")))) {
 	    section_len += line_len;
-	    if (!iscomment(line, "%%")) {
+	    if (!DSCcomment(line)) {
 		/* Do nothing */
 	    } else if (doc->pages[doc->numpages].orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
@@ -835,23 +879,38 @@ continuepage:
 
     /* Document Trailer */
 
-    doc->begintrailer = position;
-    section_len = line_len;
+    if (beginsection) {
+	doc->begintrailer = beginsection;
+	beginsection = 0;
+    } else {
+	doc->begintrailer = position;
+	section_len = line_len;
+    }
 
     preread = 1;
     while ((preread ||
 	    readline(line, sizeof line, file, &position, &line_len)) &&
-	   !iscomment(line, "%%EOF")) {
+	   !(DSCcomment(line) && iscomment(line+2, "EOF"))) {
 	if (!preread) section_len += line_len;
 	preread = 0;
-	if (!iscomment(line, "%%")) {
+	if (!DSCcomment(line)) {
 	    /* Do nothing */
 	} else if (iscomment(line+2, "Page:")) {
 	    free(gettext(line+length("%%Page:"), &next_char));
 	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
-		doc->pages[doc->numpages-1].end = position;
-		doc->pages[doc->numpages-1].len += section_len - line_len;
+		if (doc->numpages > 0) {
+		    doc->pages[doc->numpages-1].end = position;
+		    doc->pages[doc->numpages-1].len += section_len - line_len;
+		} else {
+		    if (doc->endsetup) {
+			doc->endsetup = position;
+			doc->endsetup += section_len - line_len;
+		    } else if (doc->endprolog) {
+			doc->endprolog = position;
+			doc->endprolog += section_len - line_len;
+		    }
+		}
 		goto newpage;
 	    }
 	} else if (bb_set == ATEND && iscomment(line+2, "BoundingBox:")) {
@@ -905,30 +964,44 @@ continuepage:
 	}
     }
     section_len += line_len;
-    if (iscomment(line, "%%EOF")) {
+    if (DSCcomment(line) && iscomment(line+2, "EOF")) {
 	readline(line, sizeof line, file, &position, &line_len);
 	section_len += line_len;
     }
     doc->endtrailer = position;
     doc->lentrailer = section_len - line_len;
 
+#if 0
     section_len = line_len;
     preread = 1;
     while (preread ||
 	   readline(line, sizeof line, file, &position, &line_len)) {
 	if (!preread) section_len += line_len;
 	preread = 0;
-	if (iscomment(line, "%%Page:")) {
+	if (DSCcomment(line) && iscomment(line+2, "Page:")) {
 	    free(gettext(line+length("%%Page:"), &next_char));
 	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
-		doc->pages[doc->numpages-1].end = position;
-		doc->pages[doc->numpages-1].len += doc->lentrailer +
-						   section_len - line_len;
+		if (doc->numpages > 0) {
+		    doc->pages[doc->numpages-1].end = position;
+		    doc->pages[doc->numpages-1].len += doc->lentrailer +
+						       section_len - line_len;
+		} else {
+		    if (doc->endsetup) {
+			doc->endsetup = position;
+			doc->endsetup += doc->lentrailer +
+					 section_len - line_len;
+		    } else if (doc->endprolog) {
+			doc->endprolog = position;
+			doc->endprolog += doc->lentrailer +
+					  section_len - line_len;
+		    }
+		}
 		goto newpage;
 	    }
 	}
     }
+#endif
     return doc;
 }
 
@@ -1102,56 +1175,59 @@ readline(line, size, fp, position, line_len)
 
     if (position) *position = ftell(fp);
     cp = fgets(line, size, fp);
-    *line_len = cp ? strlen(line) : 0;
-    if (iscomment(line, "%%BeginDocument:")) {
+    if (cp == NULL) line[0] = '\0';
+    *line_len = strlen(line);
+    if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
+	/* Do nothing */
+    } else if (iscomment(line+7, "Document:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndDocument")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndDocument"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginFeature:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "Feature:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndFeature")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndFeature"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginFile:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "File:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndFile")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndFile"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginFont:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "Font:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndFont")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndFont"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginProcSet:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "ProcSet:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndProcSet")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndProcSet"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginResource:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "Resource:")) {
 	strcpy(save, line+7);
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndResource")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndResource"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginData:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "Data:")) {
 	text[0] = '\0';
 	strcpy(save, line+7);
 	if (sscanf(line+length("%%BeginData:"), "%d %*s %s", &num, text) >= 1) {
@@ -1171,12 +1247,12 @@ readline(line, size, fp, position, line_len)
 	    }
 	}
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndData")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndData"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
-    } else if (iscomment(line, "%%BeginBinary:")) {
+	strcpy(line, save);
+    } else if (iscomment(line+7, "Binary:")) {
 	strcpy(save, line+7);
 	if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
 	    while (num > BUFSIZ) {
@@ -1188,11 +1264,11 @@ readline(line, size, fp, position, line_len)
 	    *line_len += num;
 	}
 	while (readline(line, size, fp, NULL, &nbytes) &&
-	       !iscomment(line, "%%EndBinary")) {
+	       !(DSCcomment(line) && iscomment(line+2, "EndBinary"))) {
 	    *line_len += nbytes;
 	}
 	*line_len += nbytes;
-	strcpy(line+2, save);
+	strcpy(line, save);
     }
     return cp;
 }
@@ -1224,7 +1300,9 @@ pscopy(from, to, begin, end)
 	fgets(line, sizeof line, from);
 	fputs(line, to);
 
-	if (iscomment(line, "%%BeginData:")) {
+	if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
+	    /* Do nothing */
+	} else if (iscomment(line+7, "Data:")) {
 	    text[0] = '\0';
 	    if (sscanf(line+length("%%BeginData:"),
 		       "%d %*s %s", &num, text) >= 1) {
@@ -1243,7 +1321,7 @@ pscopy(from, to, begin, end)
 		    fwrite(buf, sizeof (char), num, to);
 		}
 	    }
-	} else if (iscomment(line, "%%BeginBinary:")) {
+	} else if (iscomment(line+7, "Binary:")) {
 	    if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
 		while (num > BUFSIZ) {
 		    fread(buf, sizeof (char), BUFSIZ, from);
@@ -1302,7 +1380,9 @@ pscopyuntil(from, to, begin, end, comment)
 	    return cp;
 	}
 	fputs(line, to);
-	if (iscomment(line, "%%BeginData:")) {
+	if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
+	    /* Do nothing */
+	} else if (iscomment(line+7, "Data:")) {
 	    text[0] = '\0';
 	    if (sscanf(line+length("%%BeginData:"),
 		       "%d %*s %s", &num, text) >= 1) {
@@ -1321,7 +1401,7 @@ pscopyuntil(from, to, begin, end, comment)
 		    fwrite(buf, sizeof (char), num, to);
 		}
 	    }
-	} else if (iscomment(line, "%%BeginBinary:")) {
+	} else if (iscomment(line+7, "Binary:")) {
 	    if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
 		while (num > BUFSIZ) {
 		    fread(buf, sizeof (char), BUFSIZ, from);
@@ -1337,13 +1417,15 @@ pscopyuntil(from, to, begin, end, comment)
 }
 
 /*
- *	isblank -- determine whether the line contains nothing but whitespace.
+ *	blank -- determine whether the line contains nothing but whitespace.
  */
 
 static int
-isblank(line)
+blank(line)
     char *line;
 {
-    while (*line == ' ' || *line == '\t') line++;
-    return *line == '\n';
+    char *cp = line;
+
+    while (*cp == ' ' || *cp == '\t') cp++;
+    return *cp == '\n' || (*cp == '%' && (line[0] != '%' || line[1] != '%'));
 }
